﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.ComponentModel;

namespace HeuristicLab.Encodings.RealVectorEncoding {
  [Item("CMAES Strategy Parameters", "CMA-ES controls many strategy parameters that guide the search and which are combined in this class.")]
  [StorableClass]
  public sealed class CovarianceMatrixStrategyParameters : Item, INotifyPropertyChanged {
    [Storable]
    private DoubleValue sigma;
    public DoubleValue Sigma {
      get { return sigma; }
      set {
        if (sigma == value) return;
        sigma = value;
        OnPropertyChanged("Sigma");
      }
    }

    [Storable]
    private IntValue mu;
    public IntValue Mu {
      get { return mu; }
      set {
        if (mu == value) return;
        mu = value;
        OnPropertyChanged("Mu");
      }
    }

    [Storable]
    private DoubleArray weights;
    public DoubleArray Weights {
      get { return weights; }
      set {
        if (weights == value) return;
        weights = value;
        OnPropertyChanged("Weights");
      }
    }

    [Storable]
    private DoubleValue muEff;
    public DoubleValue MuEff {
      get { return muEff; }
      set {
        if (muEff == value) return;
        muEff = value;
        OnPropertyChanged("VarianceEffectiveness");
      }
    }

    [Storable]
    private DoubleValue cc;
    public DoubleValue CC {
      get { return cc; }
      set {
        if (cc == value) return;
        cc = value;
        OnPropertyChanged("CC");
      }
    }

    [Storable]
    private DoubleValue cs;
    public DoubleValue CS {
      get { return cs; }
      set {
        if (cs == value) return;
        cs = value;
        OnPropertyChanged("CS");
      }
    }

    [Storable]
    private DoubleValue c1;
    public DoubleValue C1 {
      get { return c1; }
      set {
        if (c1 == value) return;
        c1 = value;
        OnPropertyChanged("C1");
      }
    }

    [Storable]
    private DoubleValue cmu;
    public DoubleValue CMU {
      get { return cmu; }
      set {
        if (cmu == value) return;
        cmu = value;
        OnPropertyChanged("CMU");
      }
    }

    [Storable]
    private DoubleArray pc;
    public DoubleArray PC {
      get { return pc; }
      set {
        if (pc == value) return;
        pc = value;
        OnPropertyChanged("PC");
      }
    }

    [Storable]
    private DoubleArray ps;
    public DoubleArray PS {
      get { return ps; }
      set {
        if (ps == value) return;
        ps = value;
        OnPropertyChanged("PS");
      }
    }

    [Storable]
    private DoubleMatrix b;
    public DoubleMatrix B {
      get { return b; }
      set {
        if (b == value) return;
        b = value;
        OnPropertyChanged("B");
      }
    }

    [Storable]
    private DoubleArray d;
    public DoubleArray D {
      get { return d; }
      set {
        if (d == value) return;
        d = value;
        OnPropertyChanged("D");
      }
    }

    [Storable]
    private DoubleMatrix c;
    public DoubleMatrix C {
      get { return c; }
      set {
        if (c == value) return;
        c = value;
        OnPropertyChanged("C");
      }
    }

    [Storable]
    private DoubleMatrix invSqrtC;
    public DoubleMatrix InvSqrtC {
      get { return invSqrtC; }
      set {
        if (invSqrtC == value) return;
        invSqrtC = value;
        OnPropertyChanged("InvSqrtC");
      }
    }

    [Storable]
    private DoubleValue chiN;
    public DoubleValue ChiN {
      get { return chiN; }
      set {
        if (chiN == value) return;
        chiN = value;
        OnPropertyChanged("ChiN");
      }
    }

    [StorableConstructor]
    private CovarianceMatrixStrategyParameters(bool deserializing) : base(deserializing) { }
    private CovarianceMatrixStrategyParameters(CovarianceMatrixStrategyParameters original, Cloner cloner)
      : base(original, cloner) {
      this.b = cloner.Clone(original.b);
      this.c = cloner.Clone(original.c);
      this.c1 = cloner.Clone(original.c1);
      this.cc = cloner.Clone(original.cc);
      this.chiN = cloner.Clone(original.chiN);
      this.cmu = cloner.Clone(original.cmu);
      this.cs = cloner.Clone(original.cs);
      this.d = cloner.Clone(original.d);
      this.invSqrtC = cloner.Clone(original.invSqrtC);
      this.mu = cloner.Clone(original.mu);
      this.muEff = cloner.Clone(original.muEff);
      this.pc = cloner.Clone(original.pc);
      this.ps = cloner.Clone(original.ps);
      this.sigma = cloner.Clone(original.sigma);
      this.weights = cloner.Clone(original.weights);
    }
    public CovarianceMatrixStrategyParameters() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CovarianceMatrixStrategyParameters(this, cloner);
    }

    public event PropertyChangedEventHandler PropertyChanged;
    private void OnPropertyChanged(string name) {
      var handler = PropertyChanged;
      if (handler != null) handler(this, new PropertyChangedEventArgs(name));
    }
  }
}
